from download import get_problems
import pandas as pd
import json
from pathlib import Path
from markdownify import markdownify as md
import click
import re
import textwrap
import os
import template

def get_data(series):

    def as_markdown(content):
        content = md(content or "", strip=['strong'])
        long_lines = re.findall(r".*(Explanation.*)", content)
        for l in long_lines:
            content = content.replace(l, textwrap.fill(l, 80))
        return content

    def get_solution(problem_id):
        p = Path(os.getenv("LEETCODE_DATA_PATH")) / 'solutions'
        matches = list(p.glob(f"{int(problem_id):04d}*.py"))
        if not matches:
            return ""
        p = matches[0]
        with open(p, 'r') as f:
            solution = f.read()
        return solution

    open_path = Path(os.getenv('LEETCODE_DATA_PATH')) / 'neetcode.json'
    with open(open_path, 'r') as f:
        neet = json.load(f)
    neet = pd.DataFrame(neet) 
    problems = pd.DataFrame([d.to_dict() for d in get_problems()])
    neet['slug'] = neet.link.str.strip('/')
    del(neet['difficulty'])
    problems = pd.merge(problems, neet, left_on="title_slug", right_on="slug")
    problems['tags'] = problems.topic_tags.apply(lambda t: " | ".join([d.get('slug') for d in t]))
    problems['markdown'] = problems.content.apply(lambda c: as_markdown(c))
    problems.rename(columns={"question_frontend_id": "id"}, inplace=True)
    problems['solution'] = problems.id.apply(lambda i: get_solution(i))
    patterns = [
        "Arrays & Hashing"
        ,"Two Pointers"
        ,"Sliding Window"
        ,"Stack"
        ,"Binary Search"
        ,"Linked List"
        ,"Trees"
        ,"Tries"
        ,"Heap / Priority Queue"
        ,"Backtracking"
        ,"Graphs"
        ,"Advanced Graphs"
        ,"1-D Dynamic Programming"
        ,"2-D Dynamic Programming"
        ,"Greedy"
        ,"Intervals"
        ,"Math & Geometry"
        ,"Bit Manipulation"
    ]
    #patterns = pd.DataFrame(patterns, columns=['category'])
    problems = pd.merge(problems, pd.DataFrame(patterns, columns=['pattern']).reset_index(), on='pattern')
    problems = problems[(problems[series] == True) & (problems.premium != True)]
    return problems


def save_solutions(data, save_to: Path, problem_set):
    out = template.get('main_title.md').render({"problem_set": problem_set})
    keys = ['title', 'slug', 'id', 'solution', 'pattern', 'difficulty', 'tags', 'index']
    number = 0
    total = len(data)
    grouped = data[keys].groupby(['index', 'pattern', 'difficulty'])
    for i, ((pattern_id, pattern, difficulty), group) in enumerate(grouped):
        out += template.get('pattern_title.md').render({"pattern": pattern, "difficulty":difficulty})
        page = template.get("solution.md")
        for j, problem in group.iterrows():
            number += 1
            problem = problem.to_dict()
            problem["total"] = total
            problem["number"] = number
            out += page.render(problem)
    with open(save_to, "w") as f:
        f.write(out)

def save_problems(problems, save_to: Path, problem_set):
    keys = ['title', 'slug', 'id', 'markdown', 'pattern', 'difficulty', 'index']
    out = template.get('main_title.md').render({"problem_set": problem_set})
    number = 0
    grouped = problems[keys].groupby(['index', 'pattern', 'difficulty'])
    for i, ((pattern_id, pattern, difficulty), group) in enumerate(grouped):
        out += template.get('pattern_title.md').render({"pattern": pattern, "difficulty":difficulty})
        page = template.get("solution.md")
        for j, problem in group.iterrows():
            number += 1
            problem = problem.to_dict()
            problem["total"] = total
            problem["number"] = number
            out += page.render(problem)
    with open(save_to, "w") as f:
        f.write(out)

@click.command("build")
@click.option('-o', '--out', type=click.Choice(['problems', 'solutions']), multiple=True, default=['problems'])
@click.option('--problem_set', default="neetcode150", type=click.Choice(['neetcode150', 'blind75', 'all']))
def build(out, problem_set):
    data = get_data(problem_set)
    if 'problems' in out:
        save_to = Path(os.getenv("LEETCODE_BUILD_PATH")) / f'problems-{problem_set}.md'
        save_problems(data, save_to, problem_set)
    if 'solutions' in out:
        save_to = Path(os.getenv("LEETCODE_BUILD_PATH")) / f'solutions-{problem_set}.md'
        save_solutions(data, save_to, problem_set)
        print(f"saved: {save_to.absolute()}")
